# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
EzQt_Widgets package initialization.
"""
__version__ = "2.2.1"

# BUTTON WIDGETS
from .button import *

# INPUT WIDGETS
from .input import *

# LABEL WIDGETS
from .label import *

# MISC WIDGETS
from .misc import *
